%%Non-Linear Fitting Script for Initial Rate Estimation by Omar A.
%%Abdelrahman, UMass Amherst, Released 2021
% To Use This Script, an EXCEL File Named rate_dat is Required in the Same 
% Folder as the Matlab Script. Time Data is to be Placed in Column 'A' and 
% Rate Data is Placed in Column 'B' 

clear
clc

%Assigning Experimental Data to Handles
Dat = xlsread('rate_dat'); % Extract Time vs Rate data from XLS spreadsheet with Time in the first column and Rate in the second 
Time = Dat(:,1); % Time data in Minutes
Rate_exp = Dat(:,2); % Experimentally Measured Rates in umol/g.min

%Non-Linear Fitting of Non-Zero Asymptote Rate Model
Rate_model = @(beta, Time) exp(-beta(2)*Time)*(beta(1)-beta(3))+beta(3); % Define Non-Zero Asymptote Rate Model 
Beta0(1)= 500; %Initial Rate guess (umol/g.min)
Beta0(2)= 0.01; %Deactivation Rate Constant guess (1/min)
Beta0(3)= 200; %Steady State Rate guess (umol/g.min)
[BETA,R,J,COVB,MSE]  = nlinfit(Time,Rate_exp, Rate_model,Beta0); % Non-linear Fitting of Rate Prediction Based on Initial Guesses for Fitted Parameters
CI = nlparci(BETA,R,'jacobian',J); % Confidence Interval Calculation, 95% Confidence Level by Default
Rate_pred = Rate_model(BETA,[0;Time]); % Predicted Rate of Reaction Based on Fitted Parameters 

%Plotting of Residual Values
figure (2)
bar(Time,R./Rate_exp*100,2,'r'); % Bar Plot of Residuals Normalized by Experimental Rates (%)
xlabel('Time [min]','fontweight','bold'); % X Axis Label
ylabel('(Rate_e_x_p - Rate_p_r_e_d)/Rate_e_x_p [%]','fontweight','bold'); % Y Axis Label

%Plotting of Experimental and Predicted Reaction Rates
figure (1)
plot(Time,Rate_exp,'o','color','k'); % Scatter plot of experimentally measured rates
hold on 
plot([0;Time],Rate_pred,'color','b','LineWidth',1.5); % Line Plot of Predicted Rate, Extrapolated to Zero Time on Stream
formatSpec = '%.1e'; % Formatting Specification for Displaying Scientific Notation with One Decimal Place
txt = ['Initial Rate = ' num2str(round(BETA(1))) ' ' char(177) ' ' num2str(round(CI(1,2)-CI(1,1))) ' \mumol/g.min']; % Assign Text Handle to Initial Rate Estimate
txt2 = ['Deactivation Rate Constant = ' num2str(BETA(2),formatSpec) ' ' char(177) ' ' num2str(CI(2,2)-CI(2,1),formatSpec) ' 1/min']; % Assign Text Handle to Deactivation Rate Constant Estimate
txt3 = ['Steady State Rate = ' num2str(round(BETA(3))) ' ' char(177) ' ' num2str(round(CI(3,2)-CI(3,1))) ' \mumol/g.min']; % Assign Text Handle to Steady State Rate Estimate
xL=xlim; % Assigning Handle to X-Axis Limits
yL=ylim; % Assigning Handle to Y-Axis Limits
text(0.96*xL(2),0.98*yL(2),txt,'HorizontalAlignment','right','VerticalAlignment','top')
text(0.96*xL(2),0.94*yL(2),txt2,'HorizontalAlignment','right','VerticalAlignment','top')
text(0.96*xL(2),0.9*yL(2),txt3,'HorizontalAlignment','right','VerticalAlignment','top') 
title('1st order deactivation with non-zero asymptote'); % Plot Title 
xlabel('Time [min]','fontweight','bold'); % X Axis Label
ylabel('Rate [\mumol/g.min]','fontweight','bold'); % Y Axis Label